/**
 * This class represents a route for which
 * an event can be routed to.
 *
 * A route defines a step of actions to perform as well as
 * what kind of events those actions should be performed for.
 *
 * @export
 * @class Route
 */
export declare class Route {
    /**
     * The pattern used to filter out the events
     * used to trigger this route.
     *
     * @type {*}
     * @memberof Route
     */
    pattern: any;
    /**
     * Tbe internal array of actions to perform for this
     * route.
     *
     * @private
     * @type {Array<any>}
     * @memberof Route
     */
    private actions;
    /**
     * Creates an instance of Route.
     *
     * @param {*} pattern
     * @memberof Route
     */
    constructor(pattern: any);
    /**
     * Registers an action to be performed for this route.
     *
     * This method returns an instance of the route so that
     * more actions can be declared via chaining.
     *
     * Each action will be performed in the order that it is
     * declared.
     *
     * @param {(any) => any} action
     * @returns
     * @memberof Route
     */
    then(action: (any) => any): void;
    /**
     * This method determines if the pattern for
     * the route matches a given event using the
     * built-in matchers.
     *
     * @param {any} event
     * @returns
     * @memberof Route
     */
    matches(event: any): any;
    /**
     * This method triggers the actions for this route
     * to be executed by passing them the event one by one.
     *
     * @param {any} event
     * @returns Promise
     * @memberof Route
     */
    handle(event: any): any;
}
